# UNESCAP Introduction to Machine Learning and Text Mining for Trade Policy
# Lecture 2 - Demonstration
# Ben Shepherd. Ben@Developing-Trade.com





# Clear the environment and set the working directory (change to your own as appropriate)
rm(list=ls())
setwd("/link/to/your/directory/for/Lecture 2/")

# Some useful libraries; use install.packages("library name") to make sure you have them first.
library(XML)
library(tidytext)
library (tidyverse)
library(dplyr)
library(tm)
library(topicmodels)
library(textstem)
library(reshape2)
data(stop_words)

# Read XML data for the Japan-Thailand FTA, and set it up as a dataframe
text <- xmlParse(paste0(getwd(),"/tota-master/xml/pta_1.xml"))
textdf <- xmlToDataFrame(nodes = getNodeSet(text, "//article"))
articles <- nrow(textdf)
textdf$article <- c(1:articles)

# this line was added for Mac OS users, to ensure that data in column 'text' is classed as characters rather than factors
textdf$text = as.character(textdf$text)

# Tokenize the text of the agreement (one word per row)
jtfta_tokens <- unnest_tokens(textdf, word, text)

# Remove stop words and numbers
jtfta_tokens_nostop <- anti_join(jtfta_tokens, stop_words)
jtfta_tokens_nostop$word <- gsub(x=jtfta_tokens_nostop$word, pattern = "[0-9]", replacement = "")
jtfta_tokens_nostop <- jtfta_tokens_nostop[!(jtfta_tokens_nostop$word == ""),]

# Lemmatize
jtfta_tokens_nostop$word <- lemmatize_words(as.vector(jtfta_tokens_nostop$word))

# Look at the frequency of words
jtfta_tokens_nostop %>%
  count(word, sort = TRUE)
jtfta_tokens_nostop %>%
  count(word, sort = TRUE) %>%
  filter(n>50) %>%
  mutate(word = reorder(word, n)) %>%
  ggplot(aes(n, word)) +
  geom_col()

# Set up the Egypt-Turkey FTA in the same way
text <- xmlParse(paste0(getwd(),"/tota-master/xml/pta_2.xml"))
textdf <- xmlToDataFrame(nodes = getNodeSet(text, "//article"))
articles <- nrow(textdf)
textdf$article <- c(1:articles)

# this line was added for Mac OS users, to ensure that data in column 'text' is classed as characters rather than factors
textdf$text = as.character(textdf$text)
                           
etfta_tokens <- unnest_tokens(textdf, word, text)
etfta_tokens_nostop <- anti_join(etfta_tokens, stop_words)
etfta_tokens_nostop$word <- gsub(x=etfta_tokens_nostop$word, pattern = "[0-9]", replacement = "")
etfta_tokens_nostop <- etfta_tokens_nostop[!(etfta_tokens_nostop$word == ""),]
etfta_tokens_nostop$word <- lemmatize_words(as.vector(etfta_tokens_nostop$word))
etfta_tokens_nostop %>%
  count(word, sort = TRUE)
etfta_tokens_nostop %>%
  count(word, sort = TRUE) %>%
  filter(n>10) %>%
  mutate(word = reorder(word, n)) %>%
  ggplot(aes(n, word)) +
  geom_col()

# Draw a scatter plot of word frequencies in the two agreements (only words that appear in both)
frequency <- bind_rows(mutate(jtfta_tokens_nostop, agreement = "JapanThailand"), mutate(etfta_tokens_nostop, agreement = "EgyptTurkey")) %>%
  count(agreement, word) %>%
  group_by(agreement) %>%
  mutate(frequency = n / sum(n)) %>%
  select(-n) %>%
  spread(agreement, frequency) %>%
  na.omit()
ggplot(frequency, aes(x = log(JapanThailand), y = log(EgyptTurkey))) +
  geom_point() +
  geom_text(aes(label = word), check_overlap = TRUE, vjust = 1.5) +
  geom_smooth() +
  geom_abline()

# Look at TF-IDF in the Japan-Thailand agreement, by article; focus on "environmental" and check against Article 112 of the original text.
text <- xmlParse(paste0(getwd(),"/tota-master/xml/pta_1.xml"))
textdf <- xmlToDataFrame(nodes = getNodeSet(text, "//article"))

jtfta_tfidf <- jtfta_tokens_nostop %>%
  count(article, word, sort = TRUE)
jtfta_tfidf <- bind_tf_idf(jtfta_tfidf, word, article, n)
arrange(jtfta_tfidf, desc(tf_idf))
textdf[112,1]

# Plot TF-IDF scores by word for articles 112 (environment) and 21 (agriculture)
ggplot(filter(jtfta_tfidf, article == 112), aes(x=word, y=tf_idf)) +
  geom_bar(stat = "identity")
ggplot(filter(jtfta_tfidf, article == 21), aes(x=word, y=tf_idf)) +
  geom_bar(stat = "identity")

# LDA topic analysis of the Japan-Thailand agreement; assume 5 topics; plot topic terms, and check Articles 23 and 92 for relevance to their highest weighted topics.
word_counts <- count(jtfta_tokens_nostop, word, article, sort = TRUE)
articles_dtm <- cast_dtm(word_counts, article, word, n)
articles_lda <- LDA(articles_dtm, k=5, control = list(seed=1234))
topics <- tidy(articles_lda, matrix = "beta")
top_terms <- topics %>%
  group_by(topic) %>%
  top_n(10, beta) %>%
  ungroup() %>%
  arrange(topic, -beta)
top_terms %>%
  mutate(term = reorder_within(term, beta, topic)) %>%
  ggplot(aes(beta, term, fill=factor(topic))) +
  geom_col(show_legend = FALSE) +
  facet_wrap(~ topic, scales = "free") +
  scale_y_reordered()
articles <- tidy(articles_lda, matrix = "gamma")
articles <- arrange(articles, document, -gamma)
textdf[23,1]
textdf[92,1]
